//
//  ZFJ_Animation.m
//  TransitionAnimation
//
//  Created by ZFJ on 2017/6/26.
//  Copyright © 2017年 ZFJ. All rights reserved.
//

#import "ZFJ_Animation.h"

#define ScreenWidth    [UIScreen mainScreen].bounds.size.width
#define ScreenHeight   [UIScreen mainScreen].bounds.size.height

@interface ZFJ_Animation ()<UIViewControllerAnimatedTransitioning>{
    UIView *_coverView;
}

@end

@implementation ZFJ_Animation

- (instancetype)init{
    if(self == [super init]){
        self.operation = UINavigationControllerOperationNone;
    }
    return self;
}

#pragma mark - UIViewControllerAnimatedTransitioning
// This is used for percent driven interactive transitions, as well as for
// container controllers that have companion animations that might need to
// synchronize with the main animation.
- (NSTimeInterval)transitionDuration:(nullable id <UIViewControllerContextTransitioning>)transitionContext{
    return 0.4;
}
// This method can only  be a nop if the transition is interactive and not a percentDriven interactive transition.
- (void)animateTransition:(id <UIViewControllerContextTransitioning>)transitionContext{
    UIViewController *fromViewController = [transitionContext viewControllerForKey:UITransitionContextFromViewControllerKey];
    
    UIViewController *toViewController = [transitionContext viewControllerForKey:UITransitionContextToViewControllerKey];
    
    UIView *containerView = transitionContext.containerView;

    if(self.animationType == ZFJ_AnimationType1){
        
        [containerView addSubview:toViewController.view];
        
        CGRect fromViewStartFrame = [transitionContext initialFrameForViewController:fromViewController];
        CGRect toViewEndFrame = [transitionContext finalFrameForViewController:toViewController];
        CGRect fromViewEndFrame = fromViewStartFrame;
        CGRect toViewStartFrame = toViewEndFrame;
        
        if(self.operation == UINavigationControllerOperationPush){
            //PUSH
            toViewStartFrame.origin.y -= toViewEndFrame.size.height;
        }else if (self.operation == UINavigationControllerOperationPop){
            //POP
            fromViewEndFrame.origin.y -= fromViewStartFrame.size.height;
            [containerView sendSubviewToBack:toViewController.view];
        }
        //这里面 你可以自定义动画
        fromViewController.view.frame = fromViewStartFrame;
        toViewController.view.frame = toViewStartFrame;
        
        //动画
        CGFloat duration = [self transitionDuration:transitionContext];
        [UIView animateWithDuration:duration animations:^{
            fromViewController.view.frame = fromViewEndFrame;
            toViewController.view.frame = toViewEndFrame;
        } completion:^(BOOL finished) {
            [transitionContext completeTransition:YES];
        }];
    }else if(self.animationType == ZFJ_AnimationType2){
        [containerView insertSubview:toViewController.view belowSubview:fromViewController.view];

        CATransform3D t = CATransform3DIdentity;
        t = CATransform3DRotate(t, M_PI / 2.0, 0.0, 1.0, 0.0);
        t.m34 = 1.0 / -2000;

        if (self.operation == UINavigationControllerOperationPush) {
            [self setAnchorPoint:CGPointMake(1.0, 0.5) forView:toViewController.view];
            [self setAnchorPoint:CGPointMake(0.0, 0.5) forView:fromViewController.view];
        } else if (self.operation == UINavigationControllerOperationPop) {
            [self setAnchorPoint:CGPointMake(0.0, 0.5) forView:toViewController.view];
            [self setAnchorPoint:CGPointMake(1.0, 0.5) forView:fromViewController.view];
        }
        
        fromViewController.view.layer.zPosition = 2.0;
        toViewController.view.layer.zPosition = 1.0;
        toViewController.view.layer.transform = t;
        
        [UIView animateWithDuration:[self transitionDuration:transitionContext] animations:^{
            fromViewController.view.layer.transform = t;
            toViewController.view.layer.transform = CATransform3DIdentity;
        } completion:^(BOOL finished) {
            fromViewController.view.layer.zPosition = 0.0;
            toViewController.view.layer.zPosition = 0.0;
            [transitionContext completeTransition:YES];
        }];
    }else if (self.animationType == ZFJ_AnimationType3){
        UIView *fromSnapshot = [fromViewController.view snapshotViewAfterScreenUpdates:NO];
        fromSnapshot.frame = fromViewController.view.frame;
        [containerView insertSubview:fromSnapshot aboveSubview:fromViewController.view];
        [fromViewController.view removeFromSuperview];
        
        toViewController.view.frame = fromSnapshot.frame;
        [containerView insertSubview:toViewController.view belowSubview:fromSnapshot];

        CGFloat width = floorf(fromSnapshot.frame.size.width/2.0)+5.0;
        [UIView animateKeyframesWithDuration:[self transitionDuration:transitionContext] delay:0.0 options:0 animations:^{
            [UIView addKeyframeWithRelativeStartTime:0.0 relativeDuration:0.20 animations:^{
                CATransform3D fromT = CATransform3DIdentity;
                fromT.m34 = 1.0 / -2000;
                fromT = CATransform3DTranslate(fromT, 0.0, 0.0, -590.0);
                fromSnapshot.layer.transform = fromT;
                
                CATransform3D toT = CATransform3DIdentity;
                toT.m34 = 1.0 / -2000;
                toT = CATransform3DTranslate(fromT, 0.0, 0.0, -600.0);
                toViewController.view.layer.transform = toT;
            }];
            [UIView addKeyframeWithRelativeStartTime:0.20 relativeDuration:0.20 animations:^{
                if (self.operation == UINavigationControllerOperationPush) {
                    fromSnapshot.layer.transform = CATransform3DTranslate(fromSnapshot.layer.transform, -width, 0.0, 0.0);
                    toViewController.view.layer.transform = CATransform3DTranslate(toViewController.view.layer.transform, width, 0.0, 0.0);
                } else if (self.operation == UINavigationControllerOperationPop) {
                    fromSnapshot.layer.transform = CATransform3DTranslate(fromSnapshot.layer.transform, width, 0.0, 0.0);
                    toViewController.view.layer.transform = CATransform3DTranslate(toViewController.view.layer.transform, -width, 0.0, 0.0);
                }
            }];
            [UIView addKeyframeWithRelativeStartTime:0.40 relativeDuration:0.20 animations:^{
                fromSnapshot.layer.transform = CATransform3DTranslate(fromSnapshot.layer.transform, 0.0, 0.0, -200);
                toViewController.view.layer.transform = CATransform3DTranslate(toViewController.view.layer.transform, 0.0, 0.0, 500);
            }];
            [UIView addKeyframeWithRelativeStartTime:0.60 relativeDuration:0.20 animations:^{
                CATransform3D fromT = fromSnapshot.layer.transform;
                CATransform3D toT = toViewController.view.layer.transform;
                if (self.operation == UINavigationControllerOperationPush) {
                    fromT = CATransform3DTranslate(fromT, floorf(width), 0.0, 200.0);
                    toT = CATransform3DTranslate(fromT, floorf(-(width*0.03)), 0.0, 0.0);
                } else if (self.operation == UINavigationControllerOperationPop) {
                    fromT = CATransform3DTranslate(fromT, floorf(-width), 0.0, 200.0);
                    toT = CATransform3DTranslate(fromT, floorf(width*0.03), 0.0, 0.0);
                }
                fromSnapshot.layer.transform = fromT;
                toViewController.view.layer.transform = toT;
            }];
            [UIView addKeyframeWithRelativeStartTime:0.80 relativeDuration:0.20 animations:^{
                toViewController.view.layer.transform = CATransform3DIdentity;
            }];
        } completion:^(BOOL finished) {
            [fromSnapshot removeFromSuperview];
            [transitionContext completeTransition:YES];
        }];
    }
    
}

- (void)setAnchorPoint:(CGPoint)anchorPoint forView:(UIView *)view {
    CGPoint oldOrigin = view.frame.origin;
    view.layer.anchorPoint = anchorPoint;
    CGPoint newOrigin = view.frame.origin;
    
    CGPoint transition;
    transition.x = newOrigin.x - oldOrigin.x;
    transition.y = oldOrigin.y - oldOrigin.y;
    
    view.center = CGPointMake (view.center.x - transition.x, view.center.y - transition.y);
}


@end


/*
 想必大家对 UINavigationController 的过场动画再熟悉不过了. 没错! 就是那万年不变的 push 和 pop 动画: 在屏幕右侧以从右至左的姿态滑入, 再从右侧以从左至右的姿态消失于黑暗中… 当然, 我对此并没有任何贬义和不满, 毕竟 Apple 延续下来的东西是有必然的道理在里面的, 而且我们手机上系统自带的有导航栏的应用都延续了这种风格, 因此这也算是 Apple 血液里的一种基因了吧.
 好了, 为了再次一睹这种动画的风采, 我准备好了一份项目的起始文件, 可以从 https://github.com/magiclee203/NavAnimator 下载. 这里我制作了一个非常简易的图片浏览器, 是通过 UINavigationController 的 push 和 pop 来实现浏览功能的.
 
 
 首先要让 DTNavController 知道要执行过场动画了, 因此, 我们需要为其设置代理. 让 DTNavController 自己通知自己最好不过了, 所以我们将其自身设置为代理. 别忘了在设置代理前, 要先遵守 UINavigationControllerDelegate 协议.
 我们直接在 DTNavController 的 viewDidLoad 方法中来操作.
 
 
 
 
 
 */
















